from ilastik.gui.ribbons.ilastikTabBase import IlastikTabBase, TabButton

from PyQt4 import QtGui, QtCore

from ilastik.gui.iconMgr import ilastikIcons

from ilastik.gui.overlaySelectionDlg import OverlaySelectionDialog
from ilastik.gui.overlayWidget import OverlayWidget
from ilastik.modules.unsupervised_decomposition.gui.guiThread import UnsupervisedDecomposition
from ilastik.modules.unsupervised_decomposition.gui.unsupervisedSelectionDlg import UnsupervisedSelectionDlg
import ilastik.gui.volumeeditor as ve

#*******************************************************************************
# U n s u p e r v i s e d T a b                                                *
#*******************************************************************************

class UnsupervisedTab(IlastikTabBase, QtGui.QWidget):
    name = 'Unsupervised Decomposition'
    position = 4
    moduleName = "Unsupervised_Decomposition"
    
    def __init__(self, parent=None):
        IlastikTabBase.__init__(self, parent)
        QtGui.QWidget.__init__(self, parent)
        
        self._initContent()
        self._initConnects()
        
        self.overlays = None

    def on_activation(self):
        if self.ilastik.project is None:
            return
        ovs = self.ilastik._activeImage.module[self.__class__.moduleName].getOverlayRefs()
        if len(ovs) == 0:
            raw = self.ilastik._activeImage.overlayMgr["Raw Data"]
            if raw is not None:
                ovs.append(raw.getRef())
                        
        self.ilastik.labelWidget._history.volumeEditor = self.ilastik.labelWidget

        overlayWidget = OverlayWidget(self.ilastik.labelWidget, self.ilastik.project.dataMgr)
        self.ilastik.labelWidget.setOverlayWidget(overlayWidget)

        self.ilastik.labelWidget.setLabelWidget(ve.DummyLabelWidget())
        
          
                
    def on_deActivation(self):
        pass
    
    def on_otherProject(self):
        self.btnChooseOverlays.setEnabled(True)
        self.btnUnsupervisedOptions.setEnabled(True) 
        
        if self.ilastik.project.dataMgr[0].shape[-1] < 2:
            self.ilastik.ribbon.setTabEnabled(self.ilastik.ribbon.tabDict[self.name].position, False)
            if self.ilastik.ribbon.tabDict[self.name].position not in self.ilastik.ribbon.disabled_tabs:
                self.ilastik.ribbon.disabled_tabs.append(self.ilastik.ribbon.tabDict[self.name].position)
        else:
            self.ilastik.ribbon.setTabEnabled(self.ilastik.ribbon.tabDict[self.name].position, True)
            if self.ilastik.ribbon.tabDict[self.name].position in self.ilastik.ribbon.disabled_tabs:
                self.ilastik.ribbon.disabled_tabs.remove(self.ilastik.ribbon.tabDict[self.name].position)        
            
    def _initContent(self):
        tl = QtGui.QHBoxLayout()
        tl.setMargin(0)
        
        self.btnChooseOverlays      = TabButton('Select Overlay', ilastikIcons.Select)
        self.btnDecompose           = TabButton('Decompose', ilastikIcons.Play)
        self.btnUnsupervisedOptions = TabButton('Unsupervised Decomposition Options', ilastikIcons.System)

        self.btnDecompose.setEnabled(False)     
        self.btnUnsupervisedOptions.setEnabled(False)     
        
        self.btnChooseOverlays.setToolTip('Choose the overlays for unsupervised decomposition')
        self.btnDecompose.setToolTip('Perform unsupervised decomposition')
        self.btnUnsupervisedOptions.setToolTip('Select an unsupervised decomposition plugin and change settings')
        self.btnChooseOverlays.setEnabled(False)   
        
        tl.addWidget(self.btnChooseOverlays)
        tl.addWidget(self.btnDecompose)
        tl.addStretch()
        tl.addWidget(self.btnUnsupervisedOptions)
        
        self.setLayout(tl)
        
    def _initConnects(self):
        self.connect(self.btnChooseOverlays, QtCore.SIGNAL('clicked()'), self.on_btnChooseOverlays_clicked)
        self.connect(self.btnDecompose, QtCore.SIGNAL('clicked()'), self.on_btnDecompose_clicked)
        self.connect(self.btnUnsupervisedOptions, QtCore.SIGNAL('clicked()'), self.on_btnUnsupervisedOptions_clicked)
        
    def on_btnChooseOverlays_clicked(self):
        dlg = OverlaySelectionDialog(self.parent,  singleSelection = False)
        overlays = dlg.exec_()
        
        if len(overlays) > 0:
            self.overlays = overlays
            # add all overlays
            for overlay in overlays:
                ref = overlay.getRef()
                ref.setAlpha(0.4)
                self.parent.labelWidget.overlayWidget.addOverlayRef(ref)
                
            self.parent.labelWidget.repaint()
            self.btnDecompose.setEnabled(True)
        else:
            self.btnDecompose.setEnabled(False)         
        
    def on_btnDecompose_clicked(self):
        self.unsDec = UnsupervisedDecomposition(self.ilastik)
        ok = self.unsDec.checkInput(self.overlays)
        if ok is True:
            self.unsDec.start(self.overlays)
        else:
            QtGui.QMessageBox.information(self, 'Decomposition not possible', ok)

    def on_btnUnsupervisedOptions_clicked(self):
        dialog = UnsupervisedSelectionDlg(self.parent)
        answer = dialog.exec_()
        if answer != None:
            self.parent.project.dataMgr.module["Unsupervised_Decomposition"].unsupervisedMethod = answer  
